import {
  Express, NextFunction, Request, Response, Router,
} from 'express';
import { ValidationError } from 'objection';

interface Route {
  method?: 'get' | 'post' | 'delete' | 'options';
  name: string;
  path: string;
}

const routes: Route[] = [
  {
    name: 'main',
    path: '/',
  },
  {
    name: 'ping',
    path: '/ping',
  },
  {
    name: 'wikidot/interwiki',
    path: '/interwiki/:wiki_name',
  },
  {
    name: 'wikidot/random-page',
    path: '/wikidot_random_page',
  },
  {
    name: 'wikidot/api/random-page',
    path: '/api/random_page',
  },
  {
    name: 'wikidot/api/random-page',
    path: '/api/wikidot_random_page',
  },
  {
    name: 'wikidot/api/tags-list',
    path: '/_api/wikidot_tags_search/list',
  },
  {
    name: 'wikidot/api/tags-search',
    path: '/_api/wikidot_tags_search/find',
  },
  {
    name: 'wikidot/api/untranslated-list',
    path: '/_api/wanted_translations/list',
  },
  {
    name: 'wikidot/get-page',
    path: '/api/wikidot/pages/:wikiName/:pageName',
  },
  {
    name: 'int/ban-records/list',
    path: '/api/private/ban-records',
  },
  {
    name: 'int/ban-records/create',
    path: '/api/private/ban-records',
    method: 'post',
  },
  {
    name: 'alice',
    path: '/api/private/alice',
    method: 'post',
  },
  {
    name: 'pages/list',
    path: '/api/private/pages',
  },
  {
    name: 'pages/show',
    path: '/api/private/pages/:id',
  },
  {
    name: 'pages/create',
    path: '/api/private/pages',
    method: 'post',
  },
  {
    name: 'pages/update',
    path: '/api/private/pages/:id',
    method: 'post',
  },
  {
    name: 'pages/delete',
    path: '/api/private/pages/:id',
    method: 'delete',
  },
];

export default function initRoutes(app: Express): void {
  routes.forEach((route: Route) => {
    const method = route.method || 'get';
    const routeMiddleware: ExpressMiddleware[] = [];
    const routeHandler = require(`./controllers/${route.name}`).default;

    app[method](route.path, routeMiddleware, (req: Request, res: Response, next: NextFunction) => {
      Promise
        .resolve(routeHandler(req, res))
        .catch((error) => {
          if (error instanceof ValidationError) {
            return res.status(400).send({ error: 'Validation error', details: error.data });
          }
          return next(error);
        });
    });
  });
}
